/*******************************************
 * Copyright(c) 2000-2002 Mikio Hirabayashi
 *******************************************/


/* إåե */
#include <gdbm.h>
#include "aqua.h"



/*************************************************************************************************
 * GDBM
 *************************************************************************************************/


static GDBM_FILE aqdbf = NULL;
static const char *aqdbfile = NULL;


int aqdbcapture(const char *file, int writable){
  assert(file);
  if(aqdbf) aqdbrelease();
  if(writable){
    if(!(aqdbf = gdbm_open((char *)file, AQ_WRITESIZE, GDBM_WRCREAT, AQ_FILEMODE, 0))) return -1;
  } else {
    if(!(aqdbf = gdbm_open((char *)file, AQ_READSIZE, GDBM_READER, AQ_FILEMODE, 0))) return -1;
  }
  aqdbfile = file;
  return 0;
}


void aqdbrelease(void){
  if(aqdbf) gdbm_close(aqdbf);
  aqdbf = NULL;
  aqdbfile = NULL;
}


int aqdbwrite(const char *file, const char *key, const char *val, int over, const char *delim){
  GDBM_FILE dbf;
  datum dk, dc, tmp;
  int ret;
  assert(file && key && val);
  if(aqdbf && aqdbfile && !strcmp(file, aqdbfile)){
    dbf = aqdbf;
  } else {
    if(!(dbf = gdbm_open((char *)file, AQ_WRITESIZE, GDBM_WRCREAT, AQ_FILEMODE, 0))) return -1;
  }
  dk.dptr = (char *)key;
  dk.dsize = strlen(key);
  dc.dptr = (char *)val;
  dc.dsize = strlen(val);
  if(over){
    if(delim){
      tmp = gdbm_fetch(dbf, dk);
      if(tmp.dptr){
        tmp.dptr = ezrealloc(tmp.dptr, tmp.dsize + strlen(delim) + dc.dsize + 1);
        sprintf(tmp.dptr + tmp.dsize, "%s%s", delim, val);
        tmp.dsize = tmp.dsize + strlen(delim) + dc.dsize;
        ret = gdbm_store(dbf, dk, tmp, GDBM_REPLACE);
        free(tmp.dptr);
      } else {
        ret = gdbm_store(dbf, dk, dc, GDBM_REPLACE);
      }
    } else {
      ret = gdbm_store(dbf, dk, dc, GDBM_REPLACE);
    }
  } else {
    ret = gdbm_store(dbf, dk, dc, GDBM_INSERT);
  }
  if(dbf != aqdbf) gdbm_close(dbf);
  return ret;
}


char *aqdbsearch(const char *file, const char *key){
  GDBM_FILE dbf;
  datum dk, dc;
  assert(file && key);
  if(aqdbf && aqdbfile && !strcmp(file, aqdbfile)){
    dbf = aqdbf;
  } else {
    if(!(dbf = gdbm_open((char *)file, AQ_READSIZE, GDBM_READER, AQ_FILEMODE, 0))) return NULL;
  }
  dk.dptr = (char *)key;
  dk.dsize = strlen(key);
  dc = gdbm_fetch(dbf, dk);
  if(dbf != aqdbf) gdbm_close(dbf);
  if(dc.dptr){
    dc.dptr = ezrealloc(dc.dptr, dc.dsize + 1);
    dc.dptr[dc.dsize] = '\0';
    return dc.dptr;
  }
  return NULL;
}


int aqdbexists(const char *file, const char *key){
  GDBM_FILE dbf;
  datum dk;
  int ret;
  assert(file && key);
  if(aqdbf && aqdbfile && !strcmp(file, aqdbfile)){
    dbf = aqdbf;
  } else {
    if(!(dbf = gdbm_open((char *)file, AQ_READSIZE, GDBM_READER, AQ_FILEMODE, 0))) return -1;
  }
  dk.dptr = (char *)key;
  dk.dsize = strlen(key);
  ret = gdbm_exists(dbf, dk);
  if(dbf != aqdbf) gdbm_close(dbf);
  return ret ? 0 : 1;
}


int aqdbdelete(const char *file, const char *key){
  GDBM_FILE dbf;
  datum dk;
  int ret;
  assert(file && key);
  if((ret = aqdbexists(file, key)) != 0) return ret;
  if(aqdbf && aqdbfile && !strcmp(file, aqdbfile)){
    dbf = aqdbf;
  } else {
    if(!(dbf = gdbm_open((char *)file, AQ_WRITESIZE, GDBM_WRITER, AQ_FILEMODE, 0))) return -1;
  }
  dk.dptr = (char *)key;
  dk.dsize = strlen(key);
  ret = gdbm_delete(dbf, dk);
  if(dbf != aqdbf) gdbm_close(dbf);
  return ret == -1 ? 1 : 0;
}


int aqdbwritemap(const char *file, const EZMAP *map, int over, const char *delim){
  GDBM_FILE dbf;
  datum dk, dc, tmp;
  char *key, *val;
  int i, ret;
  assert(file && map);
  if(aqdbf && aqdbfile && !strcmp(file, aqdbfile)){
    dbf = aqdbf;
  } else {
    if(!(dbf = gdbm_open((char *)file, AQ_WRITESIZE, GDBM_WRCREAT, AQ_FILEMODE, 0))) return -1;
  }
  for(i = 0; i < ezmapsize(map); i++){
    key = (char *)ezmapkey(map, i);
    val = (char *)ezmapval(map, i);
    dk.dptr = key;
    dk.dsize = strlen(key);
    dc.dptr = val;
    dc.dsize = strlen(val);
    if(over){
      if(delim){
        tmp = gdbm_fetch(dbf, dk);
        if(tmp.dptr){
          tmp.dptr = ezrealloc(tmp.dptr, tmp.dsize + strlen(delim) + dc.dsize + 1);
          sprintf(tmp.dptr + tmp.dsize, "%s%s", delim, val);
          tmp.dsize = tmp.dsize + strlen(delim) + dc.dsize;
          ret = gdbm_store(dbf, dk, tmp, GDBM_REPLACE);
          free(tmp.dptr);
        } else {
          ret = gdbm_store(dbf, dk, dc, GDBM_REPLACE);
        }
      } else {
        ret = gdbm_store(dbf, dk, dc, GDBM_REPLACE);
      }
    } else {
      ret = gdbm_store(dbf, dk, dc, GDBM_INSERT);
    }
    if(ret == -1){
      if(dbf != aqdbf) gdbm_close(dbf);
      return -1;
    }
  }
  if(dbf != aqdbf) gdbm_close(dbf);
  return 0;
}


EZMAP *aqdbreadmap(const char *file){
  GDBM_FILE dbf;
  datum dk, dc, tmp;
  EZMAP *map;
  assert(file);
  if(aqdbf && aqdbfile && !strcmp(file, aqdbfile)){
    dbf = aqdbf;
  } else {
    if(!(dbf = gdbm_open((char *)file, AQ_READSIZE, GDBM_READER, AQ_FILEMODE, 0))) return NULL;
  }
  map = eznewmap();
  dk = gdbm_firstkey(dbf);
  while(dk.dptr){
    dk.dptr = ezrealloc(dk.dptr, dk.dsize + 1);
    dk.dptr[dk.dsize] = '\0';
    dc = gdbm_fetch(dbf, dk);
    if(dc.dptr){
      dc.dptr = ezrealloc(dc.dptr, dc.dsize + 1);
      dc.dptr[dc.dsize] = '\0';
      ezmapset(map, (char *)dk.dptr, (char *)dc.dptr, TRUE);
      free(dc.dptr);
    }
    tmp = gdbm_nextkey(dbf, dk);
    free(dk.dptr);
    dk.dptr = tmp.dptr;
    dk.dsize = tmp.dsize;
  }
  if(dbf != aqdbf) gdbm_close(dbf);
  return map;
}


int aqdbwritenum(const char *file, const char *key, int num, int over, int add){
  GDBM_FILE dbf;
  datum dk, dc, tmp;
  int ret;
  assert(file && key);
  if(aqdbf && aqdbfile && !strcmp(file, aqdbfile)){
    dbf = aqdbf;
  } else {
    if(!(dbf = gdbm_open((char *)file, AQ_WRITESIZE, GDBM_WRCREAT, AQ_FILEMODE, 0))) return -1;
  }
  dk.dptr = (char *)key;
  dk.dsize = strlen(key);
  dc.dptr = (char *)&num;
  dc.dsize = sizeof(num);
  if(over){
    if(add){
      tmp = gdbm_fetch(dbf, dk);
      if(tmp.dptr){
        tmp.dsize += sizeof(num);
        tmp.dptr = ezrealloc(tmp.dptr, tmp.dsize);
        *((int *)(tmp.dptr + tmp.dsize) - 1) = num;
        if((ret = gdbm_store(dbf, dk, tmp, GDBM_REPLACE)) != -1) ret = tmp.dsize / sizeof(num);
        free(tmp.dptr);
      } else {
        if((ret = gdbm_store(dbf, dk, dc, GDBM_REPLACE)) != -1) ret = 1;
      }
    } else {
      if((ret = gdbm_store(dbf, dk, dc, GDBM_REPLACE)) != -1) ret = 1;
    }
  } else {
    if((ret = gdbm_store(dbf, dk, dc, GDBM_INSERT)) != -1) ret = ret == 1 ? 0 : 1;
  }
  if(dbf != aqdbf) gdbm_close(dbf);
  return ret;
}


int aqdbwritearray(const char *file, const char *key,
                   const int *array, int anum, int over, int add){
  GDBM_FILE dbf;
  datum dk, dc, tmp;
  int i, ret, *ap;
  assert(file && key && anum >= 0);
  if(aqdbf && aqdbfile && !strcmp(file, aqdbfile)){
    dbf = aqdbf;
  } else {
    if(!(dbf = gdbm_open((char *)file, AQ_WRITESIZE, GDBM_WRCREAT, AQ_FILEMODE, 0))) return -1;
  }
  dk.dptr = (char *)key;
  dk.dsize = strlen(key);
  dc.dptr = (char *)array;
  dc.dsize = sizeof(array[0]) * anum;
  if(over){
    if(add){
      tmp = gdbm_fetch(dbf, dk);
      if(tmp.dptr){
        tmp.dsize += sizeof(array[0]) * anum;
        tmp.dptr = ezrealloc(tmp.dptr, tmp.dsize);
        ap = (int *)(tmp.dptr + tmp.dsize) - anum;
        for(i = 0; i < anum; i++){
          ap[i] = array[i];
        }
        if((ret = gdbm_store(dbf, dk, tmp, GDBM_REPLACE)) != -1)
          ret = tmp.dsize / sizeof(array[0]);
        free(tmp.dptr);
      } else {
        if((ret = gdbm_store(dbf, dk, dc, GDBM_REPLACE)) != -1) ret = anum;
      }
    } else {
      if((ret = gdbm_store(dbf, dk, dc, GDBM_REPLACE)) != -1) ret = anum;
    }
  } else {
    if((ret = gdbm_store(dbf, dk, dc, GDBM_INSERT)) != -1) ret = ret == 1 ? 0 : anum;
  }
  if(dbf != aqdbf) gdbm_close(dbf);
  return ret;
}


int *aqdbsearcharray(const char *file, const char *key, int *sp){
  GDBM_FILE dbf;
  datum dk, dc;
  assert(file && key && sp);
  if(aqdbf && aqdbfile && !strcmp(file, aqdbfile)){
    dbf = aqdbf;
  } else {
    if(!(dbf = gdbm_open((char *)file, AQ_READSIZE, GDBM_READER, AQ_FILEMODE, 0))) return NULL;
  }
  dk.dptr = (char *)key;
  dk.dsize = strlen(key);
  dc = gdbm_fetch(dbf, dk);
  if(dbf != aqdbf) gdbm_close(dbf);
  if(dc.dptr) *sp = dc.dsize / sizeof(int);
  return (int *)dc.dptr;
}


AQARRAY *aqdbsearcharrays(const char *file, EZCONT *keys){
  GDBM_FILE dbf;
  datum dk, dc;
  AQARRAY *arrays;
  int i;
  assert(file && keys);
  if(aqdbf && aqdbfile && !strcmp(file, aqdbfile)){
    dbf = aqdbf;
  } else {
    if(!(dbf = gdbm_open((char *)file, AQ_READSIZE, GDBM_READER, AQ_FILEMODE, 0))) return NULL;
  }
  arrays = ezmalloc(ezcontsize(keys) * sizeof(arrays[0]));
  for(i = 0; i < ezcontsize(keys); i++){
    dk.dptr = (char *)ezcontval(keys, i);
    dk.dsize = strlen(dk.dptr);
    dc = gdbm_fetch(dbf, dk);
    if((arrays[i].array = (int *)dc.dptr) != NULL)
      arrays[i].size = dc.dsize / sizeof(arrays[i].array[0]);
  }
  if(dbf != aqdbf) gdbm_close(dbf);
  return arrays;
}


void aqdbarraysfree(AQARRAY *arrays, int asnum){
  int i;
  assert(arrays && asnum >= 0);
  for(i = 0; i < asnum; i++){
    free(arrays[i].array);
  }
  free(arrays);
}


int aqdbwritecontnum(const char *file, const EZCONT *keys, int *vals, int over, int add){
  GDBM_FILE dbf;
  datum dk, dc, tmp;
  int i, err;
  assert(file && keys && vals);
  if(aqdbf && aqdbfile && !strcmp(file, aqdbfile)){
    dbf = aqdbf;
  } else {
    if(!(dbf = gdbm_open((char *)file, AQ_WRITESIZE, GDBM_WRCREAT, AQ_FILEMODE, 0))) return -1;
  }
  for(i = 0; i < ezcontsize(keys); i++){
    dk.dptr = (char *)ezcontval(keys, i);
    dk.dsize = strlen(dk.dptr);
    dc.dptr = (char *)(vals + i);
    dc.dsize = sizeof(vals[0]);
    err = FALSE;
    if(over){
      if(add){
        tmp = gdbm_fetch(dbf, dk);
        if(tmp.dptr){
          tmp.dsize += sizeof(vals[0]);
          tmp.dptr = ezrealloc(tmp.dptr, tmp.dsize);
          *((int *)(tmp.dptr + tmp.dsize) - 1) = vals[i];
          if(gdbm_store(dbf, dk, tmp, GDBM_REPLACE) == -1) err = TRUE;
          free(tmp.dptr);
        } else {
          if(gdbm_store(dbf, dk, dc, GDBM_REPLACE) == -1) err = TRUE;
        }
      } else {
        if(gdbm_store(dbf, dk, dc, GDBM_REPLACE) == -1) err = TRUE;
      }
    } else {
      if(gdbm_store(dbf, dk, dc, GDBM_INSERT) == -1) err = TRUE;
    }
    if(err){
      if(dbf != aqdbf) gdbm_close(dbf);
      return -1;
    }
  }
  if(dbf != aqdbf) gdbm_close(dbf);
  return 0;
}


EZCONT *aqdbsearchcont(const char *file, const EZCONT *cont){
  GDBM_FILE dbf;
  datum dk, dc;
  EZCONT *res;
  int i;
  if(aqdbf && aqdbfile && !strcmp(file, aqdbfile)){
    dbf = aqdbf;
  } else {
    if(!(dbf = gdbm_open((char *)file, AQ_READSIZE, GDBM_READER, AQ_FILEMODE, 0))) return NULL;
  }
  res = eznewcont();
  for(i = 0; i < ezcontsize(cont); i++){
    dk.dptr = (char *)ezcontval(cont, i);
    dk.dsize = strlen(dk.dptr);
    dc = gdbm_fetch(dbf, dk);
    if(dc.dptr){
      dc.dptr = ezrealloc(dc.dptr, dc.dsize + 1);
      dc.dptr[dc.dsize] = '\0';
      ezcontpush(res, dc.dptr);
      free(dc.dptr);
    }
  }
  if(dbf != aqdbf) gdbm_close(dbf);
  return res;
}


EZCONT *aqdbdiffcont(const char *file, const EZCONT *cont, int be){
  GDBM_FILE dbf;
  datum dk;
  EZCONT *res;
  int i;
  if(aqdbf && aqdbfile && !strcmp(file, aqdbfile)){
    dbf = aqdbf;
  } else {
    if(!(dbf = gdbm_open((char *)file, AQ_READSIZE, GDBM_READER, AQ_FILEMODE, 0))) return NULL;
  }
  res = eznewcont();
  for(i = 0; i < ezcontsize(cont); i++){
    dk.dptr = (char *)ezcontval(cont, i);
    dk.dsize = strlen(dk.dptr);
    if(gdbm_exists(dbf, dk)){
      if(be) ezcontpush(res, dk.dptr);
    } else {
      if(!be) ezcontpush(res, dk.dptr);
    }
  }
  if(dbf != aqdbf) gdbm_close(dbf);
  return res;
}


int aqdbreorganize(const char *file){
  GDBM_FILE dbf;
  int ret;
  assert(file);
  if(aqdbf && aqdbfile && !strcmp(file, aqdbfile)){
    dbf = aqdbf;
  } else {
    if(!(dbf = gdbm_open((char *)file, AQ_READSIZE, GDBM_READER, AQ_FILEMODE, 0))) return -1;
  }
  if(dbf != aqdbf) gdbm_close(dbf);
  if(aqdbf && aqdbfile && !strcmp(file, aqdbfile)){
    dbf = aqdbf;
  } else {
    if(!(dbf = gdbm_open((char *)file, AQ_WRITESIZE, GDBM_WRITER, AQ_FILEMODE, 0))) return -1;
  }
  ret = gdbm_reorganize(dbf);
  if(dbf != aqdbf) gdbm_close(dbf);
  return ret;
}
