/*******************************************
 * Copyright(c) 2000-2002 Mikio Hirabayashi
 *******************************************/


/* إåե */
#include "easy.h"



/*************************************************************************************************
 * ѥ롼
 *************************************************************************************************/


const char *ezprogname = "libeasy";


void (*ezmyfatal)(const char *mesg) = NULL;
int ezfatalcode = 1;
void ezfatalerror(const char *mesg){
  if(ezmyfatal) ezmyfatal(mesg);
  fflush(stdout);
  if(mesg) fprintf(stderr, "%s: %s\n", ezprogname, mesg);
  exit(ezfatalcode);
}


int ezdumplog(const char *filename, const char *mesg){
  FILE *OUT;
  assert(filename && mesg);
  if(!(OUT = fopen(filename, "a"))) return FALSE;
  fprintf(OUT, "%s: %s\n", ezprogname, mesg);
  return fclose(OUT) == 0 ? TRUE : FALSE;
}


void *ezmalloc(int size){
  char *p;
  assert(size >= 0);
  if(!(p = malloc(size))) ezfatalerror("out of memory");
  return (void *)p;
}


void *ezrealloc(void *ptr, int size){
  char *p;
  assert(size >= 0);
  if(!(p = realloc(ptr, size)) && size > 1) ezfatalerror("out of memory");
  return (void *)p;
}


char *ezstrdup(const char *str){
  char *p;
  assert(str);
  p = ezmalloc(strlen(str) + 1);
  sprintf(p, "%s", str);
  return p;
}


char *ezstrcat(char *str, ...){
  va_list ap;
  char *p;
  int headlen, taillen;
  va_start(ap, str);
  headlen = str ? strlen(str) : 0;
  while((p = va_arg(ap, char *)) != NULL){
    taillen = strlen(p);
    str = ezrealloc(str, headlen + taillen + 1);
    sprintf(str + headlen, "%s", p);
    headlen += taillen;
  }
  va_end(ap);
  return str;
}


char *ezsprintf(char *str, const char *format, ...){
  va_list ap;
  int headlen, taillen, n;
  char miniform[EZ_NUMBUFSIZ+3], numbuf[EZ_NUMBUFSIZ], *wp, *s, c;
  double d;
  va_start(ap, format);
  assert(format);
  headlen = str ? strlen(str) : 0;
  for(; *format != '\0'; format++){
    if(*format == '%'){
      format++;
      miniform[0] = '%';
      wp = miniform + 1;
      while(strchr("1234567890+- .", *format)){
        *(wp++) = *(format++);
      }
      *(wp++) = *format;
      *wp = '\0';
      switch(*format){
      case 's':
        s = va_arg(ap, char *);
        taillen = strlen(s);
        str = ezrealloc(str, headlen + taillen + 1);
        strcpy(str + headlen, s);
        headlen += taillen;
        break;
      case 'c':
        c = (char)va_arg(ap, int);
        str = ezrealloc(str, headlen + 2);
        *(str + headlen) = c;
        *(str + headlen + 1) = '\0';
        headlen++;
        break;
      case 'd': case 'i': case 'o': case 'u': case 'x': case 'X':
        n = va_arg(ap, int);
        sprintf(numbuf, miniform, n);
        taillen = strlen(numbuf);
        str = ezrealloc(str, headlen + taillen + 1);
        strcpy(str + headlen, numbuf);
        headlen += taillen;
        break;
      case 'e': case 'E': case 'f': case 'g':
        d = va_arg(ap, double);
        sprintf(numbuf, miniform, d);
        taillen = strlen(numbuf);
        str = ezrealloc(str, headlen + taillen + 1);
        strcpy(str + headlen, numbuf);
        headlen += taillen;
        break;
      case '%':
        str = ezrealloc(str, headlen + 2);
        *(str + headlen) = '%';
        *(str + headlen + 1) = '\0';
        headlen++;
        break;
      default:
        break;
      }
    } else {
      str = ezrealloc(str, headlen + 2);
      *(str + headlen) = *format;
      *(str + headlen + 1) = '\0';
      headlen++;
    }
  }
  va_end(ap);
  return str;
}


int ezstrpure(const char *str, const char *elem){
  assert(str && elem);
  while(*str != '\0'){
    if(!strchr(elem, *(str++))) return FALSE;
  }
  return TRUE;
}


int ezstrspc(const char *str){
  assert(str);
  return ezstrpure(str, EZ_SPACECHARS);
}


int ezfwmatch(const char *str, const char *key){
  int len, i;
  assert(str && key);
  len = strlen(key);
  for(i = 0; i < len; i++){
    if(str[i] != key[i] || str[i] == '\0') return FALSE;
  }
  return TRUE;
}


int ezfwimatch(const char *str, const char *key){
  int len, i;
  assert(str && key);
  len = strlen(key);
  for(i = 0; i < len; i++){
    if(tolower(str[i]) != tolower(key[i]) || str[i] == '\0') return FALSE;
  }
  return TRUE;
}


int ezrwmatch(const char *str, const char *key){
  int slen, klen, i;
  assert(str && key);
  slen = strlen(str);
  klen = strlen(key);
  for(i = 1; i <= klen; i++){
    if(str[slen-i] != key[klen-i] || i > slen) return FALSE;
  }
  return TRUE;
}


int ezrwimatch(const char *str, const char *key){
  int slen, klen, i;
  assert(str && key);
  slen = strlen(str);
  klen = strlen(key);
  for(i = 1; i <= klen; i++){
    if(tolower(str[slen-i]) != tolower(key[klen-i]) || i > slen) return FALSE;
  }
  return TRUE;
}


int ezstrchop(char *str){
  int n;
  assert(str);
  n = strlen(str);
  for(n--; n >= 0; n--){
    if(strchr(EZ_LFCHARS, str[n])){
      str[n] = '\0';
    } else {
      break;
    }
  }
  return n + 1;
}


int ezstrcchop(char *str, int end){
  char *p;
  assert(str);
  if((p = strrchr(str, end)) != NULL){
    *p = '\0';
    return p - str;
  }
  return strlen(str);
}


int ezstrtrim(char *str){
  int n;
  assert(str);
  n = strlen(str);
  for(n--; n >= 0; n--){
    if(strchr(EZ_SPACECHARS, str[n])){
      str[n] = '\0';
    } else {
      break;
    }
  }
  return n + 1;
}


char *ezstrftrim(const char *str){
  assert(str);
  while(*str != '\0' && strchr(EZ_SPACECHARS, *str)){
    str++;
  }
  return (char *)str;
}


char *ezstrupper(char *str){
  char *rv;
  assert(str);
  rv = str;
  while(*str != '\0'){
    *str = toupper(*str);
    str++;
  }
  return rv;
}


char *ezstrlower(char *str){
  char *rv;
  assert(str);
  rv = str;
  while(*str != '\0'){
    *str = tolower(*str);
    str++;
  }
  return rv;
}


char *ezstrrepl(char *str, const char *bef, const char *aft){
  int i, j;
  assert(str && bef && aft);
  for(i = 0; str[i] != '\0'; i++){
    for(j = 0; bef[j] != '\0' && aft[j] != '\0'; j++){
      if(str[i] == bef[j]) str[i] = aft[j];
    }
  }
  return str;
}


char *ezstrdiv(char **index, const char *delim){
  char *token, *p;
  assert(index && delim);
  p = token = *index;
  if(!token) return NULL;
  while(!strchr(delim, *p) && *p != '\0'){
    p++;
  }
  if(*p == '\0'){
    *index = NULL;
    return token;
  }
  *p = '\0';
  *index = p + 1;
  return token;
}


char *ezstrtok(char **index, const char *delim){
  char *token;
  assert(index && delim);
  if(!*index) return NULL;
  while((token = ezstrdiv(index, delim)) != NULL && strlen(token) < 1){
    continue;
  }
  return token;
}


int ezsplit(char *str, const char *delim, char **tokens, int limit){
  int i;
  char *index;
  assert(delim && tokens && limit >= 0);
  if(!str) return 0;
  index = str;
  for(i = 0; i < limit && index; i++){
    tokens[i] = ezstrdiv(&index, delim);
  }
  return i;
}


int ezcntchr(const char *str, const char *focus){
  int n = 0;
  assert(str && focus);
  for(; *str != '\0'; str++){
    if(strchr(focus, *str)) n++;
  }
  return n;
}


int ezstricmp(const char *s1, const char *s2){
  assert(s1 && s2);
  while(*s1 != '\0'){
    if(*s2 == '\0' || tolower(*s1) > tolower(*s2)) return 1;
    if(tolower(*s1) < tolower(*s2)) return -1;
    s1++;
    s2++;
  }
  return *s2 == '\0' ? 0 : -1;
}


char *ezobjdup(const char *obj, int len){
  char *p;
  int i;
  assert(obj && len >= 0);
  p = ezmalloc(len);
  for(i = 0; i < len; i++){
    p[i] = obj[i];
  }
  return p;
}


char *ezobjcat(char *head, int headlen, const char *tail, int taillen){
  int i;
  assert(head && tail && headlen >= 0 && taillen >= 0);
  head = ezrealloc(head, headlen + taillen);
  for(i = 0; i < taillen; i++){
    *(head + headlen + i) = tail[i];
  }
  return head;
}


char *ezreadstream(FILE *IN, int *sp){
  char *buf;
  int bufsiz, i, c;
  assert(IN);
  buf = ezmalloc(bufsiz = EZ_MEMBUFSIZ);
  for(i = 0; (c = fgetc(IN)) != EOF; i++){
    if(i + 1 >= bufsiz) buf = ezrealloc(buf, bufsiz *= 2);
    buf[i] = c;
  }
  buf[i] = '\0';
  if(sp) *sp = i;
  rewind(IN);
  return buf;
}


char *ezreaddiv(FILE *IN, const char *delim){
  char *buf;
  int bufsiz, i, c;
  assert(IN && delim);
  buf = ezmalloc(bufsiz = EZ_MEMBUFSIZ);
  for(i = 0; (c = fgetc(IN)) != EOF; i++){
    if(i + 1 >= bufsiz) buf = ezrealloc(buf, bufsiz *= 2);
    buf[i] = c;
    if(strchr(delim, c)){
      i++;
      break;
    }
  }
  buf[i] = '\0';
  if(i > 0) return buf;
  free(buf);
  return NULL;
}


char *ezreadfile(const char *filename, int *sp){
  FILE *IN;
  char *buf;
  assert(filename);
  if(!(IN = fopen(filename, "r"))) return NULL;
  buf = ezreadstream(IN, sp);
  if(fclose(IN) == EOF) return NULL;
  return buf;
}


int ezfnprint(FILE *OUT, const char *str, int width, int pad, int over){
  int i;
  assert(OUT && str);
  for(i = 0; str[i] != '\0' && (over || i < width); i++){
    fputc(str[i], OUT);
  }
  for(; i < width; i++){
    fputc(pad, OUT);
  }
  return i;
}


int ezascpurify(char *str, int del, const char *expt){
  char *start, *p;
  assert(str && expt);
  start = str;
  for(p = str; *p != '\0'; p++){
    if(!strchr(expt, *p) && ((*p >= 0 && *p <= ' ') || (*p == 0x7f))){
      if(del != EOF){
        *str = del;
        str++;
      }
    } else {
      *str = *p;
      str++;
    }
  }
  *str = '\0';
  return str - start;
}


int ezlarge(int a, int b){
  return a > b ? a : b;
}


int ezsmall(int a, int b){
  return a < b ? a : b;
}


int ezmax(int num, ...){
  va_list ap;
  int i, max;
  va_start(ap, num);
  max = INT_MIN;
  for(i = 0; i < num; i++){
    max = ezlarge(va_arg(ap, int), max);
  }
  va_end(ap);
  return max;
}


int ezmin(int num, ...){
  va_list ap;
  int i, max;
  va_start(ap, num);
  max = INT_MAX;
  for(i = 0; i < num; i++){
    max = ezsmall(va_arg(ap, int), max);
  }
  va_end(ap);
  return max;
}


int ezpow(int a, int b){
  int val = 1;
  assert(b >= 0);
  while(b-- > 0){
    val *= a;
  }
  return val;
}


double eztrunc(double a, int b){
  int d;
  if(b >= 0){
    d = (int)a;
    a -= d;
    b += 1;
    a *= ezpow(10, b);
    a = (int)a - (int)a % 10;
    a /= ezpow(10, b);
    a += d;
  } else {
    b = -b - 1;
    a /= ezpow(10, b);
    a = (int)a - (int)a % 10;
    a *= ezpow(10, b);
  }
  return a;
}


double ezraise(double a, int b){
  int d;
  if(b >= 0){
    d = (int)a;
    a -= d;
    b += 1;
    a *= ezpow(10, b);
    a = (int)a - (int)a % 10 + 10;
    a /= ezpow(10, b);
    a += d;
  } else {
    b = -b - 1;
    a /= ezpow(10, b);
    a = (int)a - (int)a % 10 + 10;
    a *= ezpow(10, b);
  }
  return a;
}


double ezround(double a, int b){
  int d;
  if(b >= 0){
    d = (int)a;
    a -= d;
    b += 1;
    a *= ezpow(10, b);
    a = (int)a - (int)a % 10 + ((int)a % 10 >= 5 ? 10 : 0);
    a /= ezpow(10, b);
    a += d;
  } else {
    b = -b - 1;
    a /= ezpow(10, b);
    a = (int)a - (int)a % 10 + ((int)a % 10 >= 5 ? 10 : 0);
    a *= ezpow(10, b);
  }
  return a;
}


int ezxdigit(int c){
  if(c >= '0' && c <= '9') return c - '0';
  if(c >= 'a' && c <= 'f') return c - 'a' + 10;
  if(c >= 'A' && c <= 'F') return c - 'A' + 10;
  return -1;
}


void ezqsortnums(int *nums, int size){
  qsort(nums, size, sizeof(nums[0]), ezpnumcmp);
}


int ezpnumcmp(const void *a, const void *b){
  assert(a && b);
  return *(int *)a - *(int *)b;
}


void ezuniqnums(int *nums, int *sp){
  int i, j;
  assert(nums && sp);
  ezqsortnums(nums, *sp);
  j = 0;
  for(i = 0; i < *sp; i++){
    if(j == 0 || nums[j-1] != nums[i]) nums[j++] = nums[i];
  }
  *sp = j;
}


int ezrand(int max){
  if(max < 1) return 0;
  return rand() / (RAND_MAX / max + 1) + 1;
}


char *ezrandstr(int len, const char *tbl){
  int tbllen, i;
  char *str;
  assert(len >= 0 && tbl);
  tbllen = strlen(tbl);
  str = ezmalloc(len + 1);
  for(i = 0; i < len; i++){
    str[i] = tbl[ezrand(tbllen)-1];
  }
  str[i] = '\0';
  return str;
}



/*************************************************************************************************
 * ƥʴط
 *************************************************************************************************/


EZCONT *eznewcont(void){
  EZCONT *cont;
  cont = ezmalloc(sizeof(*cont));
  cont->bufsiz = EZ_MEMBUFSIZ;
  cont->array = ezmalloc(sizeof(cont->array[0]) * cont->bufsiz);
  cont->size = 0;
  return cont;
}


void ezdelcont(EZCONT *cont){
  int i;
  assert(cont);
  for(i = 0; i < cont->size; i++){
    free(cont->array[i]);
  }
  free(cont->array);
  free(cont);
}


EZCONT *ezdupcont(const EZCONT *cont){
  EZCONT *nc;
  int i;
  assert(cont);
  nc = eznewcont();
  for(i = 0; i < ezcontsize(cont); i++){
    ezcontpush(nc, ezcontval(cont, i));
  }
  return nc;
}


EZCONT *ezdiffcont(const EZCONT *a, const EZCONT *b, int be){
  EZCONT *nc;
  int i;
  assert(a && b);
  nc = eznewcont();
  for(i = 0; i < ezcontsize(b); i++){
    if(ezcontsearch(a, ezcontval(b, i))){
      if(be) ezcontpush(nc, ezcontval(b, i));
    } else {
      if(!be) ezcontpush(nc, ezcontval(b, i));
    }
  }
  return nc;
}


void ezcontpush(EZCONT *cont, const char *str){
  assert(cont && str);
  if(cont->size + 1 >= cont->bufsiz)
    cont->array = ezrealloc(cont->array, sizeof(cont->array[0]) * (cont->bufsiz *= 2));
  cont->array[cont->size] = ezstrdup(str);
  (cont->size)++;
}


char *ezcontpop(EZCONT *cont){
  assert(cont);
  if(cont->size <= 0) return NULL;
  (cont->size)--;
  if(cont->size < cont->bufsiz / 2 && cont->size >= EZ_MEMBUFSIZ)
    cont->array = ezrealloc(cont->array, sizeof(cont->array[0]) * (cont->bufsiz = cont->size + 1));
  return cont->array[cont->size];
}


char *ezcontshift(EZCONT *cont){
  int i;
  char *tmp;
  assert(cont);
  if(cont->size <= 0) return NULL;
  tmp = cont->array[0];
  for(i = 0; i < cont->size - 1; i++){
    cont->array[i] = cont->array[i+1];
  }
  (cont->size)--;
  if(cont->size < cont->bufsiz / 2 && cont->size >= EZ_MEMBUFSIZ)
    cont->array = ezrealloc(cont->array, sizeof(cont->array[0]) * (cont->bufsiz = cont->size + 1));
  return tmp;
}


int ezcontsize(const EZCONT *cont){
  assert(cont);
  return cont->size;
}


const char *ezcontval(const EZCONT *cont, int num){
  assert(cont && num >= 0 && num < cont->size);
  return cont->array[num];
}


const char *ezcontsearch(const EZCONT *cont, const char *str){
  int i;
  assert(cont && str);
  for(i = 0; i < ezcontsize(cont); i++){
    if(!strcmp(ezcontval(cont, i), str)) return ezcontval(cont, i);
  }
  return NULL;
}


void ezqsortcont(EZCONT *cont){
  assert(cont);
  qsort(cont->array, cont->size, sizeof(cont->array[0]), ezpstrcmp);
}


const char *ezcontbsearch(const EZCONT *cont, const char *str){
  char **val;
  assert(cont && str);
  val = bsearch(&str, cont->array, cont->size, sizeof(cont->array[0]), ezpstrcmp);
  return val ? *val : NULL;
}


int ezpstrcmp(const void *a, const void *b){
  assert(a && *(char **)a && b && *(char **)b);
  return strcmp(*(char **)a, *(char **)b);
}


void ezuniqcont(EZCONT *cont){
  EZCONT *tmp;
  char *last, *val;
  int i;
  assert(cont);
  ezqsortcont(cont);
  tmp = eznewcont();
  last = NULL;
  while((val = ezcontpop(cont)) != NULL){
    if(!last || strcmp(val, last)){
      ezcontpush(tmp, val);
    }
    free(last);
    last = val;
  }
  free(last);
  for(i = ezcontsize(tmp) - 1; i >= 0; i--){
    ezcontpush(cont, ezcontval(tmp, i));
  }
  ezdelcont(tmp);
}


EZMAP *eznewmap(void){
  EZMAP *map;
  map = ezmalloc(sizeof(*map));
  map->bufsiz = EZ_MEMBUFSIZ;
  map->array = ezmalloc(sizeof(map->array[0]) * map->bufsiz);
  map->size = 0;
  return map;
}


void ezdelmap(EZMAP *map){
  int i;
  assert(map);
  for(i = 0; i < map->size; i++){
    ezdelmapelem(&(map->array[i]));
  }
  free(map->array);
  free(map);
}


int ezmapset(EZMAP *map, const char *key, const char *val, int repl){
  int isout;
  assert(map && key && val);
  if(!repl && ezmapget(map, key)) return FALSE;
  isout = ezmapout(map, key);
  if(map->size + 1 >= map->bufsiz)
    map->array = ezrealloc(map->array, sizeof(map->array[0]) * (map->bufsiz *= 2));
  eznewmapelem(&(map->array[map->size]), key, val);
  (map->size)++;
  ezqsortmap(map);
  return isout ? FALSE : TRUE;
}


void ezmapstore(EZMAP *map, const char **pairs){
  int i;
  assert(map && pairs);
  for(i = 0; pairs[i]; i += 2){
    assert(pairs[i+1]);
    if(map->size + 1 >= map->bufsiz)
      map->array = ezrealloc(map->array, sizeof(map->array[0]) * (map->bufsiz *= 2));
    eznewmapelem(&(map->array[map->size]), pairs[i], pairs[i+1]);
    (map->size)++;
  }
  ezqsortmap(map);
}


const char *ezmapget(const EZMAP *map, const char *key){
  EZMAPELEM *elem;
  assert(map && key);
  elem = bsearch(&key, map->array, map->size, sizeof(map->array[0]), ezmapelemcmp);
  return elem ? elem->val : NULL;
}


int ezmapout(EZMAP *map, const char *key){
  EZMAPELEM *elem;
  assert(map && key);
  elem = bsearch(&key, map->array, map->size, sizeof(map->array[0]), ezmapelemcmp);
  if(!elem) return FALSE;
  ezdelmapelem(elem);
  ezqsortmap(map);
  (map->size)--;
  if(map->size < map->bufsiz / 2 && map->size >= EZ_MEMBUFSIZ)
    map->array = ezrealloc(map->array, sizeof(map->array[0]) * (map->bufsiz = map->size));
  return TRUE;
}


int ezmapsize(const EZMAP *map){
  assert(map);
  return map->size;
}


const char *ezmapkey(const EZMAP *map, int num){
  assert(map && num >= 0 && num < map->size);
  return map->array[num].key;
}


const char *ezmapval(const EZMAP *map, int num){
  assert(map && num >= 0 && num < map->size);
  return map->array[num].val;
}


void ezqsortmap(EZMAP *map){
  assert(map);
  qsort(map->array, map->size, sizeof(map->array[0]), ezmapelemcmp);
}


int ezmapelemcmp(const void *a, const void *b){
  const EZMAPELEM *p1, *p2;
  assert(a && b);
  p1 = (const EZMAPELEM *)a;
  p2 = (const EZMAPELEM *)b;
  if(p1->key == NULL && p2->key == NULL) return 0;
  if(p1->key == NULL && p2->key != NULL) return 1;
  if(p1->key != NULL && p2->key == NULL) return -1;
  return strcmp(p1->key, p2->key);
}


void eznewmapelem(EZMAPELEM *elem, const char *key, const char *val){
  assert(elem && key && val);
  elem->key = ezstrdup(key);
  elem->val = ezstrdup(val);
}


void ezdelmapelem(EZMAPELEM *elem){
  assert(elem);
  free(elem->key);
  elem->key = NULL;
  free(elem->val);
  elem->val = NULL;
}



/*************************************************************************************************
 * CSVط
 *************************************************************************************************/


char **ezreadcsv(FILE *IN, int *sp){
  char *sbuf, **fbuf;
  int *ibuf, sbufsiz, ibufsiz, si, ii, pv, c, eol, sep, quote, be, i;
  assert(IN && sp);
  sbuf = ezmalloc(sbufsiz = EZ_MEMBUFSIZ);
  ibuf = ezmalloc((ibufsiz = EZ_ARRAYBUFSIZ) * sizeof(ibuf[0]));
  si = ii = pv = 0;
  quote = be = FALSE;
  for(;;){
    c = fgetc(IN);
    eol = sep = FALSE;
    if(pv == -1) pv = si;
    if(si + 1 >= sbufsiz) sbuf = ezrealloc(sbuf, sbufsiz *= 2);
    if(c == EOF){
      eol = TRUE;
    } else {
      be = TRUE;
      if(quote){
        if(c == '"'){
          c = fgetc(IN);
          if(c == EOF || eziseol(IN, c)){
            quote = FALSE;
            eol = TRUE;
          } else if(c == '"'){
            sbuf[si++] = '"';
          } else if(c == ','){
            quote = FALSE;
            sep = TRUE;
          } else {
            quote = FALSE;
            ungetc(c, IN);
          }
        } else {
          sbuf[si++] = c;
        }
      } else {
        if(eziseol(IN, c)){
          eol = TRUE;
        } else if(c == '"'){
          quote = TRUE;
        } else if(c == ','){
          sep = TRUE;
        } else {
          sbuf[si++] = c;
        }
      }
    }
    if(sep || eol){
      sbuf[si++] = '\0';
      if(ii + 1 >= ibufsiz) ibuf = ezrealloc(ibuf, (ibufsiz *= 2) * sizeof(ibuf[0]));
      ibuf[ii++] = pv;
      pv = -1;
    }
    if(eol) break;
  }
  if(!be){
    *sp = 0;
    free(sbuf);
    free(ibuf);
    return NULL;
  }
  *sp = ii;
  fbuf = ezmalloc(ii * sizeof(fbuf[0]));
  for(i = 0; i < ii; i++){
    fbuf[i] = sbuf + ibuf[i];
  }
  free(ibuf);
  return fbuf;
}


void ezfreecsv(char **csvbuf){
  free(csvbuf[0]);
  free(csvbuf);
}


int eziseol(FILE *IN, int c){
  int eol;
  eol = (c == '\r' || c == '\n');
  if(c == '\r'){
    c = fgetc(IN);
    if(c != '\n' && c != EOF) ungetc(c, IN);
  }
  return eol;
}



/*************************************************************************************************
 * ִط
 *************************************************************************************************/


time_t eztime(void){
  time_t t;
  if((t = time(NULL)) < 0) ezfatalerror("cannot get current time");
  return t;
}


void ezthen(time_t t, int *yp, int *mp, int *dp, int *hourp, int *minp, int *secp, int local){
  struct tm *ts;
  ts = local ? localtime(&t) : gmtime(&t);
  if(yp) *yp = ts->tm_year + 1900;
  if(mp) *mp = ts->tm_mon + 1;
  if(dp) *dp = ts->tm_mday;
  if(hourp) *hourp = ts->tm_hour;
  if(minp) *minp = ts->tm_min;
  if(secp) *secp = ts->tm_sec;
}


time_t ezwhen(int *yp, int *mp, int *dp, int *hourp, int *minp, int *secp, int offset, int local){
  time_t t;
  struct tm *ts;
  t = eztime() + offset;
  ts = local ? localtime(&t) : gmtime(&t);
  if(yp) *yp = ts->tm_year + 1900;
  if(mp) *mp = ts->tm_mon + 1;
  if(dp) *dp = ts->tm_mday;
  if(hourp) *hourp = ts->tm_hour;
  if(minp) *minp = ts->tm_min;
  if(secp) *secp = ts->tm_sec;
  return t;
}


time_t eznow(int *yp, int *mp, int *dp, int *hourp, int *minp, int *secp){
  return ezwhen(yp, mp, dp, hourp, minp, secp, 0, TRUE);
}


time_t ezmktime(int y, int m, int d, int hour, int min, int sec, int local){
  struct tm tsr;
  tsr.tm_sec = sec;
  tsr.tm_min = min + (local ? 0 : ezjetlag());
  tsr.tm_hour = hour;
  tsr.tm_mday = d;
  tsr.tm_mon = m - 1;
  tsr.tm_year = y - 1900;
  tsr.tm_isdst = -1;
  return mktime(&tsr);
}


int ezjetlag(void){
  int ly, lm, ld, lhour, lmin, gy, gm, gd, ghour, gmin, lag;
  ezwhen(&ly, &lm, &ld, &lhour, &lmin, NULL, 0, TRUE);
  ezwhen(&gy, &gm, &gd, &ghour, &gmin, NULL, 0, FALSE);
  lag = (lhour * 60 + lmin) - (ghour * 60 + gmin);
  if(ly > gy){
    lag += 24 * 60;
  } else if(ly < gy){
    lag -= 24 * 60;
  } else if(lm > gm){
    lag += 24 * 60;
  } else if(lm < gm){
    lag -= 24 * 60;
  } else if(ld > gd){
    lag += 24 * 60;
  } else if(ld < gd){
    lag -= 24 * 60;
  }
  return lag;
}


const char *ezdatestr(time_t t, int local){
  struct tm *ts;
  static char buf[EZ_DATEBUFSIZ], *wp;
  int lag;
  if(local){
    ts = localtime(&t);
    strftime(buf, EZ_DATEBUFSIZ - 1, "%a, %d %b %Y %H:%M:%S ", ts);
    wp = buf + strlen(buf);
    lag = ezjetlag();
    sprintf(wp, "%+03d%02d", lag / 60, lag % 60);
  } else {
    ts = gmtime(&t);
    strftime(buf, EZ_DATEBUFSIZ, "%a, %d %b %Y %H:%M:%S GMT", ts);
  }
  return buf;
}


time_t ezstrmktime(const char *str){
  char monthbuf[EZ_DATEBUFSIZ], lagbuf[EZ_DATEBUFSIZ];
  struct tm tse;
  int lag;
  if(strlen(str) >= EZ_DATEBUFSIZ) return -1;
  lag = 0;
  if(ezstrpure(str, "0123456789")) return atoi(str);
  if(str[3] == ','){
    if(sscanf(str, "%*s %d %s %d %d:%d:%d %s",
              &(tse.tm_mday), monthbuf, &(tse.tm_year), &(tse.tm_hour), &(tse.tm_min),
              &(tse.tm_sec), lagbuf) != 7) return -1;
    lag = atoi(lagbuf);
    tse.tm_year -= 1900;
  } else if(str[3] == ' '){
    if(sscanf(str, "%*s %s %d %d:%d:%d %d",
              monthbuf, &(tse.tm_mday), &(tse.tm_hour), &(tse.tm_min), &(tse.tm_sec),
              &(tse.tm_year)) != 6) return -1;
    tse.tm_year -= 1900;
  } else if(isalpha((int)str[3])){
    if(sscanf(str, "%*s %d-%3s-%d %d:%d:%d %s",
              &(tse.tm_mday), monthbuf, &(tse.tm_year), &(tse.tm_hour), &(tse.tm_min),
              &(tse.tm_sec), lagbuf) != 7) return -1;
    lag = atoi(lagbuf);
  } else {
    return -1;
  }
  if(ezstricmp(monthbuf, "jan") == 0){
    tse.tm_mon = 0;
  } else if(ezstricmp(monthbuf, "feb") == 0){
    tse.tm_mon = 1;
  } else if(ezstricmp(monthbuf, "mar") == 0){
    tse.tm_mon = 2;
  } else if(ezstricmp(monthbuf, "apr") == 0){
    tse.tm_mon = 3;
  } else if(ezstricmp(monthbuf, "may") == 0){
    tse.tm_mon = 4;
  } else if(ezstricmp(monthbuf, "jun") == 0){
    tse.tm_mon = 5;
  } else if(ezstricmp(monthbuf, "jul") == 0){
    tse.tm_mon = 6;
  } else if(ezstricmp(monthbuf, "aug") == 0){
    tse.tm_mon = 7;
  } else if(ezstricmp(monthbuf, "sep") == 0){
    tse.tm_mon = 8;
  } else if(ezstricmp(monthbuf, "oct") == 0){
    tse.tm_mon = 9;
  } else if(ezstricmp(monthbuf, "nov") == 0){
    tse.tm_mon = 10;
  } else if(ezstricmp(monthbuf, "dec") == 0){
    tse.tm_mon = 11;
  }
  tse.tm_min += ezjetlag() - ((lag / 100) * 60 + (lag % 100));
  tse.tm_isdst = -1;
  return(mktime(&tse));
}



/*************************************************************************************************
 * ʸط
 *************************************************************************************************/


EZCODETYPE ezcharcode(const char *str){
  const unsigned char *s;
  EZCODETYPE hypo = EZ_CASCII;
  int i, brk;
  assert(str);
  s = (const unsigned char *)str;
  for(i = 0; s[i] != '\0'; i++){
    if(s[i] == 0x1b){
      if(hypo == EZ_CEUC ||  hypo == EZ_CSJIS) return EZ_CERR;
      hypo = EZ_CJUNET;
    } else if((s[i] <= 0x20 && !strchr(EZ_SPACECHARS, s[i])) || s[i] == 0x7f) {
      return EZ_CERR;
    } else if(s[i] >= 0x80){
      if(hypo == EZ_CJUNET) return EZ_CERR;
      if(hypo != EZ_CSJIS) hypo = EZ_CEUC;
      if(s[i] <= 0xa0 && s[i] != 0x8f && s[i] != 0x8e) hypo = EZ_CSJIS;
    }
  }
  if(hypo == EZ_CEUC){
    for(i = 0, brk = FALSE; s[i] != '\0' && !brk; i++){
      switch(ezeuccharset(s[i])){
      case 0: continue;
      case 1: case 2:
        i++;
        if(s[i] < 0xa0){
          hypo = EZ_CSJIS;
          brk = TRUE;
          break;
        }
        break;
      case 3:
        i++;
        if(s[i++] < 0xa0 || s[i] < 0xa0){
          hypo = EZ_CSJIS;
          brk = TRUE;
        }
        break;
      default:
        hypo = EZ_CSJIS;
        brk = TRUE;
        break;
      }
    }
  }
  if(hypo == EZ_CSJIS){
    for(i = 0; s[i] != '\0'; i++){
      switch(ezsjischarset(s[i])){
      case 0: case 2: continue;
      case 1:
        i++;
        if(!(s[i] >= 0x40 && s[i] <= 0x7e) && !(s[i] >= 0x80 && s[i] <= 0xfc)) return EZ_CERR;
        break;
      default:
        return EZ_CERR;
      }
    }
  }
  return hypo;
}


EZLINETYPE ezlinefeed(const char *str){
  EZLINETYPE hypo = EZ_LERR;
  int i;
  for(i = 0; str[i] != '\0'; i++){
    switch(str[i]){
    case '\r':
      if(hypo == EZ_LUNIX) return EZ_LERR;
      if(str[i+1] == '\n'){
        if(hypo == EZ_LMAC) return EZ_LERR;
        hypo = EZ_LDOS;
      } else {
        hypo = EZ_LMAC;
      }
      break;
    case '\n':
      if(hypo == EZ_LMAC) return EZ_LERR;
      if(hypo == EZ_LDOS){
        if(str[i-1] != '\r') return EZ_LERR;
      } else {
        hypo = EZ_LUNIX;
      }
      break;
    default:
      break;
    }
  }
  if(hypo == EZ_LERR) hypo = EZ_LUNIX;
  return hypo;
}


int ezeuccharset(unsigned char c){
  if(c == 0x8e) return 2;
  if(c == 0x8f) return 3;
  if(c <= 0x7f) return 0;
  if(c >= 0xa0 && c <= 0xff) return 1;
  return -1;
}


int ezeucbytes(unsigned char c){
  switch(ezeuccharset(c)){
  case 0: return 1;
  case 1: return 2;
  case 2: return 2;
  case 3: return 3;
  }
  return 1;
}


int ezsjischarset(unsigned char c){
  if(c <= 0x7f) return 0;
  if(c >= 0xa1 && c <= 0xdf) return 2;
  if((c >= 0x81 && c <= 0x9f) || (c >= 0xe0 && c <= 0xef)) return 1;
  return -1;
}


int ezsjisbytes(unsigned char c){
  switch(ezsjischarset(c)){
  case 0: return 1;
  case 1: return 2;
  case 2: return 1;
  }
  return 1;
}


char *ezeuctojunet(const char *str){
  const unsigned char *euc;
  unsigned char *junet, c1, c2;
  int bufsiz, ei, ji, g, err;
  assert(str);
  euc = (const unsigned char *)str;
  junet = ezmalloc(bufsiz = strlen(str) + 1);
  ji = 0;
  g = 0;
  err = FALSE;
  for(ei = 0; euc[ei] != '\0'; ei++){
    while(ji + EZ_ESCBUFSIZ * 2 + 1 >= bufsiz){
      junet = ezrealloc(junet, bufsiz *= 2);
    }
    switch(ezeuccharset(euc[ei])){
    case 0:
      if(g != 0) ji += sprintf((char *)junet+ji, "%s", EZ_ESCASCII);
      g = 0;
      junet[ji++] = euc[ei];
      break;
    case 1:
      if(g != 1) ji += sprintf((char *)junet+ji, "%s", EZ_ESCNEWKANJI);
      g = 1;
      c1 = euc[ei++]; c2 = euc[ei];
      if(euc[ei] == '\0') err = TRUE;
      ezeucg2g3tojunet(&c1, &c2);
      junet[ji++] = c1; junet[ji++] = c2;
      break;
    case 2:
      if(g != 2) ji += sprintf((char *)junet+ji, "%s", EZ_ESCKANA);
      g = 2;
      ei++;
      if(euc[ei] == '\0') err = TRUE;
      junet[ji++] = euc[ei] - 0xA1 + 0x21;
      break;
    case 3:
      if(g != 3) ji += sprintf((char *)junet+ji, "%s", EZ_ESCEXKANJI);
      g = 3;
      ei++;
      if(euc[ei] == '\0') err = TRUE;
      c1 = euc[ei++]; c2 = euc[ei];
      if(euc[ei] == '\0') err = TRUE;
      ezeucg2g3tojunet(&c1, &c2);
      junet[ji++] = c1; junet[ji++] = c2;
      break;
    default:
      err = TRUE;
      break;
    }
    if(err) break;
  }
  if(g != 0) ji += sprintf((char *)junet+ji, "%s", EZ_ESCASCII);
  junet[ji] = '\0';
  return (char *)junet;
}


char *ezjunettoeuc(const char *str){
  const unsigned char *junet;
  unsigned char *euc, c1, c2;
  int bufsiz, ji, ei, g, err;
  assert(str);
  junet = (const unsigned char *)str;
  euc = ezmalloc(bufsiz = strlen(str) + 1);
  ei = 0;
  g = 0;
  err = FALSE;
  for(ji = 0; junet[ji] != '\0'; ji++){
    while(ji + 3 + 1 >= bufsiz){
      euc = ezrealloc(euc, bufsiz *= 2);
    }
    ezstepjunetesc(junet, &ji, &g);
    switch(g){
    case 0:
      euc[ei++] = junet[ji];
      break;
    case 1:
      if(junet[ji] == '\0') err = TRUE;
      c1 = junet[ji++]; c2 = junet[ji];
      if(junet[ji] == '\0') err = TRUE;
      ezjunetg2g3toeuc(&c1, &c2);
      euc[ei++] = c1; euc[ei++] = c2;
      break;
    case 2:
      euc[ei++] = 0x8e;
      if(junet[ji] == '\0') err = TRUE;
      euc[ei++] = junet[ji] - 0x21 + 0xa1;
      break;
    case 3:
      euc[ei++] = 0x8f;
      c1 = junet[ji++]; c2 = junet[ji];
      if(junet[ji] == '\0') err = TRUE;
      ezjunetg2g3toeuc(&c1, &c2);
      euc[ei++] = c1; euc[ei++] = c2;
      break;
    default:
      err = TRUE;
      break;
    }
    if(err) break;
  }
  euc[ei] = '\0';
  return (char *)euc;
}


char *ezeuctosjis(const char *str){
  const unsigned char *euc;
  unsigned char *sjis, c1, c2;
  int bufsiz, ei, si, err;
  assert(str);
  euc = (const unsigned char *)str;
  sjis = ezmalloc(bufsiz = strlen(str) + 1);
  si = 0;
  err = FALSE;
  for(ei = 0; euc[ei] != '\0'; ei++){
    while(si + 2 + 1 >= bufsiz){
      sjis = ezrealloc(sjis, bufsiz *= 2);
    }
    switch(ezeuccharset(euc[ei])){
    case 0:
      sjis[si++] = euc[ei];
      break;
    case 1:
      c1 = euc[ei++]; c2 = euc[ei];
      if(euc[ei] == '\0') err = TRUE;
      ezeucg2tosjis(&c1, &c2);
      sjis[si++] = c1; sjis[si++] = c2;
      break;
    case 2:
      ei++;
      if(euc[ei] == '\0') err = TRUE;
      sjis[si++] = euc[ei];
      break;
    case 3:
      break;
    default:
      err = TRUE;
      break;
    }
    if(err) break;
  }
  sjis[si] = '\0';
  return (char *)sjis;
}


char *ezsjistoeuc(const char *str){
  const unsigned char *sjis;
  unsigned char *euc, c1, c2;
  int bufsiz, si, ei, err;
  assert(str);
  sjis = (const unsigned char *)str;
  euc = ezmalloc(bufsiz = strlen(str) + 1);
  ei = 0;
  err = FALSE;
  for(si = 0; sjis[si] != '\0'; si++){
    while(ei + 2 + 1 >= bufsiz){
      euc = ezrealloc(euc, bufsiz *= 2);
    }
    switch(ezsjischarset(sjis[si])){
    case 0:
      euc[ei++] = sjis[si];
      break;
    case 1:
      c1 = sjis[si++]; c2 = sjis[si];
      if(sjis[si] == '\0') err = TRUE;
      ezsjisg2toeuc(&c1, &c2);
      euc[ei++] = c1; euc[ei++] = c2;
      break;
    case 2:
      euc[ei++] = 0x8e;
      euc[ei++] = sjis[si];
      break;
    default:
      err = TRUE;
      break;
    }
    if(err) break;
  }
  euc[ei] = '\0';
  return (char *)euc;
}


char *ezjunettosjis(const char *str){
  const unsigned char *junet;
  unsigned char *sjis, c1, c2;
  int bufsiz, ji, si, g, err;
  assert(str);
  junet = (const unsigned char *)str;
  sjis = ezmalloc(bufsiz = strlen(str) + 1);
  si = 0;
  g = 0;
  err = FALSE;
  for(ji = 0; junet[ji] != '\0'; ji++){
    while(ji + 2 + 1 >= bufsiz){
      sjis = ezrealloc(sjis, bufsiz *= 2);
    }
    ezstepjunetesc(junet, &ji, &g);
    switch(g){
    case 0:
      sjis[si++] = junet[ji];
      break;
    case 1:
      c1 = junet[ji++]; c2 = junet[ji];
      if(junet[ji] == '\0') err = TRUE;
      ezjunetg2tosjis(&c1, &c2);
      sjis[si++] = c1; sjis[si++] = c2;
      break;
    case 2:
      sjis[si++] = junet[ji] - 0x21 + 0xa1;
      break;
    case 3:
      break;
    default:
      err = TRUE;
      break;
    }
    if(err) break;
  }
  sjis[si] = '\0';
  return (char *)sjis;
}


char *ezsjistojunet(const char *str){
  const unsigned char *sjis;
  unsigned char *junet, c1, c2;
  int bufsiz, si, ji, g, err;
  assert(str);
  sjis = (const unsigned char *)str;
  junet = ezmalloc(bufsiz = strlen(str) + 1);
  ji = 0;
  g = 0;
  err = FALSE;
  for(si = 0; sjis[si] != '\0'; si++){
    while(ji + EZ_ESCBUFSIZ * 2 + 1 >= bufsiz){
      junet = ezrealloc(junet, bufsiz *= 2);
    }
    switch(ezsjischarset(sjis[si])){
    case 0:
      if(g != 0) ji += sprintf((char *)junet+ji, "%s", EZ_ESCASCII);
      g = 0;
      junet[ji++] = sjis[si];
      break;
    case 1:
      if(g != 1) ji += sprintf((char *)junet+ji, "%s", EZ_ESCNEWKANJI);
      g = 1;
      c1 = sjis[si++]; c2 = sjis[si];
      if(sjis[si] == '\0') err = TRUE;
      ezsjisg2tojunet(&c1, &c2);
      junet[ji++] = c1; junet[ji++] = c2;
      break;
    case 2:
      if(g != 2) ji += sprintf((char *)junet+ji, "%s", EZ_ESCKANA);
      g = 2;
      junet[ji++] = sjis[si] - 0xA1 + 0x21;
      break;
    case 3:
      break;
    default:
      err = TRUE;
      break;
    }
    if(err) break;
  }
  if(g != 0) ji += sprintf((char *)junet+ji, "%s", EZ_ESCASCII);
  junet[ji] = '\0';
  return (char *)junet;
}


void ezeucg2g3tojunet(unsigned char *c1, unsigned char *c2){
  *c1 &= 0x7f;
  *c2 &= 0x7f;
}


void ezstepjunetesc(const unsigned char *str, int *ip, int *gp){
  assert(str && ip && gp);
  str += *ip;
  if(ezfwmatch((char *)str, EZ_ESCASCII)){
    *ip += strlen(EZ_ESCASCII);
    *gp = 0;
  } else if(ezfwmatch((char *)str, EZ_ESCLATIN)){
    *ip += strlen(EZ_ESCLATIN);
    *gp = 0;
  } else if(ezfwmatch((char *)str, EZ_ESCLATIN_B)){
    *ip += strlen(EZ_ESCLATIN_B);
    *gp = 0;
  } else if(ezfwmatch((char *)str, EZ_ESCKANA)){
    *ip += strlen(EZ_ESCKANA);
    *gp = 2;
  } else if(ezfwmatch((char *)str, EZ_ESCOLDKANJI)){
    *ip += strlen(EZ_ESCOLDKANJI);
    *gp = 1;
  } else if(ezfwmatch((char *)str, EZ_ESCNEWKANJI)){
    *ip += strlen(EZ_ESCNEWKANJI);
    *gp = 1;
  } else if(ezfwmatch((char *)str, EZ_ESCNEWKANJI_B)){
    *ip += strlen(EZ_ESCNEWKANJI_B);
    *gp = 1;
  } else if(ezfwmatch((char *)str, EZ_ESCEXKANJI)){
    *ip += strlen(EZ_ESCEXKANJI);
    *gp = 3;
  }
}


void ezjunetg2g3toeuc(unsigned char *c1, unsigned char *c2){
  *c1 |= 0x80;
  *c2 |= 0x80;
}


void ezeucg2tosjis(unsigned char *c1, unsigned char *c2){
  if((*c1 % 2) ==0){
    *c2 -= 0x02;
  } else {
    *c2 -= 0x61;
    if(*c2 > 0x7e)
      ++ *c2;
  }
  if(*c1 < 0xdf){
    ++ *c1;
    *c1 /= 2;
    *c1 += 0x30;
  } else {
    ++ *c1;
    *c1 /= 2;
    *c1 += 0x70;
  }
}


void ezsjisg2toeuc(unsigned char *c1, unsigned char *c2){
  if(*c2 < 0x9f){
    if(*c1 < 0xa0){
      *c1 -= 0x81;
      *c1 *= 2;
      *c1 += 0xa1;
    } else {
      *c1 -= 0xe0;
      *c1 *= 2;
      *c1 += 0xdf;
    }
    if(*c2 > 0x7f)
      -- *c2;
    *c2 += 0x61;
  } else {
    if(*c1 < 0xa0){
      *c1 -= 0x81;
      *c1 *= 2;
      *c1 += 0xa2;
    } else {
      *c1 -= 0xe0;
      *c1 *= 2;
      *c1 += 0xe0;
    }
    *c2 += 2;
  }
}


void ezjunetg2tosjis(unsigned char *c1, unsigned char *c2){
  if((*c1 % 2) == 0)
    *c2 += 0x7d;
  else
    *c2 += 0x1f;
  if(*c2 > 0x7e)
    ++ *c2;
  if(*c1 < 0x5f){
    ++ *c1;
    *c1 /= 2;
    *c1 += 0x70;
  } else {
    ++ *c1;
    *c1 /= 2;
    *c1 += 0xb0;
  }
}


void ezsjisg2tojunet(unsigned char *c1, unsigned char *c2){
  if(*c2 < 0x9f){
    if(*c1 < 0xa0){
      *c1 -= 0x81;
      *c1 *= 2;
      *c1 += 0x21;
    } else {
      *c1 -= 0xe0;
      *c1 *= 2;
      *c1 += 0x5f;
    }
    if(*c2 > 0x7f)
      -- *c2;
    *c2 -= 0x1f;
  } else {
    if(*c1 < 0xa0){
      *c1 -= 0x81;
      *c1 *= 2;
      *c1 += 0x22;
    } else {
      *c1 -= 0xe0;
      *c1 *= 2;
      *c1 += 0x60;
    }
    *c2 -= 0x7e;
  }
}


char *ezescape(const char *str, const char **pairs){
  char *buf;
  int si, bi, pi, bufsiz, klen, hit;
  assert(str && pairs);
  if(!str) return NULL;
  buf = ezmalloc(bufsiz = strlen(str) + 1);
  bi = 0;
  for(si = 0; str[si] != '\0';){
    hit = FALSE;
    for(pi = 0; pairs[pi] != NULL; pi += 2){
      if(ezfwmatch(str + si, pairs[pi])){
        klen = strlen(pairs[pi+1]);
        while(bi + klen + 1 >= bufsiz){
          buf = ezrealloc(buf, bufsiz *= 2);
        }
        strcpy(buf + bi, pairs[pi+1]);
        bi += klen;
        si += strlen(pairs[pi]);
        hit = TRUE;
        break;
      }
    }
    if(!hit){
      if(bi + 1 >= bufsiz) buf = ezrealloc(buf, bufsiz *= 2);
      buf[bi++] = str[si++];
    }
  }
  buf[bi] = '\0';
  return buf;
}


char *ezclangescape(const char *str){
  const char *pairs[] = {
    "\n", "\\n", "\t", "\\t", "\v", "\\v", "\b", "\\b", "\r", "\\r", "\f", "\\f",
    "\a", "\\a", "\\", "\\\\", "?", "\\?", "'", "\\'", "\"", "\\\"", NULL
  };
  assert(str);
  return ezescape(str, pairs);
}


char *ezclangunescape(const char *str){
  const char *pairs[] = {
    "\\n", "\n", "\\t", "\t", "\\v", "\v", "\\b", "\v", "\\r", "\r", "\\f", "\f",
    "\\a", "\a", "\\\\", "\\", "\\?", "?", "\\'", "'", "\\\"", "\"", NULL
  };
  assert(str);
  return ezescape(str, pairs);
}


char *ezlineescape(const char *str, EZLINETYPE type){
  const char *unixpairs[] = { "\r\n", "\n", "\r", "\n", "\n", "\n", NULL };
  const char *dospairs[] = { "\r\n", "\r\n", "\r", "\r\n", "\n", "\r\n", NULL };
  const char *macpairs[] = { "\r\n", "\r", "\r", "\r", "\n", "\r", NULL };
  assert(str && (type == EZ_LUNIX || type == EZ_LDOS || type == EZ_LMAC));
  switch(type){
  case EZ_LUNIX: return ezescape(str, unixpairs);
  case EZ_LDOS: return ezescape(str, dospairs);
  case EZ_LMAC: return ezescape(str, macpairs);
  default: break;
  }
  return NULL;
}


char *ezeuclinefold(const char *str, int limit){
  char *esc, *wp;
  int i, len, tlen;
  wp = esc = ezmalloc(strlen(str) * 2 + 1);
  len = tlen = 0;
  for(i = 0; str[i] != '\0'; i++, tlen--, len++){
    if(str[i] == '\n') len = 0;
    if(tlen < 1){
      tlen = ezeucbytes(str[i]);
      if(len + tlen > limit){
        *(wp++) = '\n';
        len = 0;
      }
    }
    *(wp++) = str[i];
  }
  *wp = '\0';
  return esc;
}



/*************************************************************************************************
 * Webط
 *************************************************************************************************/


void ezshowmime(const char *charset){
  const char *date;
  char *key, *val;
  EZMAP *map;
  int i;
  date = ezdatestr(eztime(), FALSE);
  printf("Content-Type: text/html");
  if(charset) printf("; charset=%s", charset);
  printf("\r\n");
  printf("Cache-Control: no-cache, must-revalidate\r\n");
  printf("Pragma: no-cache\r\n");
  printf("Last-Modified: %s\r\n", date);
  printf("Expires: %s\r\n", date);
  if((map = ezsetcookie(NULL, NULL)) != NULL){
    for(i = 0; i < ezmapsize(map); i++){
      key = ezurlencode(ezmapkey(map, i));
      val = ezurlencode(ezmapval(map, i));
      printf("Set-Cookie: %s=%s\r\n", key, val);
      free(key);
      free(val);
    }
  }
  printf("\r\n");
  fflush(stdout);
}


EZMAP *ezsetcookie(const char *name, const char *val){
  static EZMAP *map = NULL;
  if(name){
    if(!map) map = eznewmap();
    if(val){
      ezmapset(map, name, val, TRUE);
    } else {
      ezmapout(map, name);
    }
  }
  return map;
}


const char *ezgetcookie(const char *name){
  static EZMAP *map = NULL;
  char *tmp, *params[EZ_MAXPARAMS], *pair[2];
  int pnum, i;
  assert(name);
  if(!map && (tmp = getenv("HTTP_COOKIE")) != NULL){
    tmp = ezstrdup(tmp);
    pnum = ezsplit(tmp, ";", params, EZ_MAXPARAMS);
    map = eznewmap();
    for(i = 0; i < pnum; i++){
      if(ezsplit(params[i], "=", pair, 2) != 2) continue;
      pair[0] = ezstrftrim(pair[0]);
      if(!ezurldecode(pair[0]) || !ezurldecode(pair[1])) continue;
      ezmapset(map, pair[0], pair[1], FALSE);
    }
    free(tmp);
  }
  return map ? ezmapget(map, name) : NULL;
}


EZMAP *ezcgimap(void){
  char *query, *tmp, *params[EZ_MAXPARAMS], *pair[2];
  int clen, pnum, i;
  EZMAP *map;
  query = NULL;
  if((tmp = getenv("REQUEST_METHOD")) != NULL && !strcmp(tmp, "GET") &&
     (tmp = getenv("QUERY_STRING")) != NULL){
    query = ezstrdup(tmp);
  } else if((tmp = getenv("REQUEST_METHOD")) != NULL && !strcmp(tmp, "POST") &&
            (tmp = getenv("CONTENT_TYPE")) != NULL &&
            !ezstricmp(tmp, "application/x-www-form-urlencoded") &&
            (tmp = getenv("CONTENT_LENGTH")) != NULL){
    clen = atoi(tmp);
    query = ezmalloc(clen + 1);
    for(i = 0; i < clen; i++){
      if((query[i] = fgetc(stdin)) == EOF) return NULL;
    }
    query[i] = '\0';
  }
  if(!query) return NULL;
  pnum = ezsplit(query, "&", params, EZ_MAXPARAMS);
  map = eznewmap();
  for(i = 0; i < pnum; i++){
    if(ezsplit(params[i], "=", pair, 2) != 2) continue;
    if(!ezurldecode(pair[0]) || !ezurldecode(pair[1])) continue;
    ezmapset(map, pair[0], pair[1], FALSE);
  }
  free(query);
  return map;
}


char *ezurlencode(const char *str){
  char *p, *wp;
  if(!str) return NULL;
  p = wp = ezmalloc(strlen(str) * 3 + 1);
  while(*str != '\0'){
    if(isalnum((int)*str) || strchr("-_.!~*'()/", *str)){
      *(wp++) = *str;
    } else if(*str == ' '){
      *(wp++) = '+';
    } else {
      wp += sprintf(wp, "%%%02X", (int)*(unsigned char *)str);
    }
    str++;
  }
  *wp = '\0';
  return p;
}


char *ezurldecode(char *str){
  const char *hex = "1234567890abcdefABCDEF";
  char *wp, *rv;
  if(!str) return NULL;
  wp = rv = str;
  for(; *str != '\0'; wp++, str++){
    if(*str == '%'){
      str++;
      if(!strchr(hex, *str)) return NULL;
      *str -= (*str >= 'a') ? 'a' - 'A' : 0;
      *wp = (*str >= 'A' ? ((*str & 0xdf) - 'A') + 10 : (*str - '0')) * 0x10; str++;
      if(!strchr(hex, *str)) return NULL;
      *str -= (*str >= 'a') ? 'a' - 'A' : 0;
      *wp += (*str >= 'A' ? ((*str & 0xdf) - 'A') + 10 : (*str - '0'));
      if(*wp == '\0') return NULL;
    } else if(*str == '+'){
      *wp = ' ';
    } else {
      *wp = *str;
    }
  }
  *wp = '\0';
  return rv;
}


int ezreadform(EZFORMDATA *data, int limit){
  char *tmp, *sub, *bound, *rawdata, *begin, *token, *val;
  int clen, c, ri, di;
  assert(data && limit > 0);
  if(!(tmp = getenv("CONTENT_LENGTH")) || (clen = atoi(tmp)) < 1 ||
     !(tmp = getenv("CONTENT_TYPE")) || !ezfwimatch(tmp, "multipart/form-data")) return -1;
  if(!(sub = strstr(tmp, "boundary="))) return -1;
  bound = ezstrcat(NULL, "\r\n--", sub + strlen("boundary="), NULL);
  ezstrchop(bound);
  rawdata = ezmalloc(clen);
  for(ri = 0; ri < clen; ri++){
    if((c = fgetc(stdin)) == EOF) return -1;
    rawdata[ri] = c;
  }
  if(!ezfwmatch(rawdata, bound + 2)){
    free(bound);
    free(rawdata);
    return -1;
  }
  ri = strlen(bound);
  for(di = 0; di < limit; di++){
    data[di].name = data[di].file = data[di].type = data[di].data = NULL;
    data[di].size = -1;
    begin = rawdata + ri;
    while(ri < clen && !ezfwmatch(rawdata + ri, bound)){
      ri++;
    }
    if((data[di].data = strstr(begin, "\r\n\r\n")) != NULL){
      data[di].data[0] = '\0';
      data[di].data += 4;
      data[di].size = rawdata + ri - data[di].data;
      while((token = ezstrdiv(&begin, "\n")) != NULL){
        ezstrchop(token);
        if(ezfwimatch(token, "Content-Disposition:")){
          val = ezstrftrim(token + strlen("Content-Disposition:"));
          while((token = ezstrdiv(&val, ";")) != NULL){
            token = ezstrftrim(token);
            if(ezfwmatch(token, "name=")){
              token += strlen("name=");
              ezstrrepl(token, "\"", "\n");
              ezstrtrim(token);
              data[di].name = ezstrftrim(token);
            } else if(ezfwmatch(token, "filename=")){
              token += strlen("filename=");
              ezstrrepl(token, "\"", "\n");
              ezstrtrim(token);
              data[di].file = ezstrftrim(token);
            }
          }
        } else if(ezfwimatch(token, "Content-Type:")){
          data[di].type = ezstrftrim(token + strlen("Content-Type:"));
        }
      }
    }
    if(ri >= clen || !ezfwmatch(rawdata + ri + strlen(bound), "\r\n")){
      di++;
      break;
    }
    ri += strlen(bound) + 2;
  }
  free(bound);
  return di;
}


EZFORMDATA *ezgetform(EZFORMDATA *data, int size, const char *name){
  int i;
  assert(data && name);
  for(i = 0; i < size; i++){
    if(data[i].name && !strcmp(data[i].name, name)) return data + i;
  }
  return NULL;
}


char *ezgetformstr(EZFORMDATA *data, int size, const char *name){
  char *buf;
  int i;
  assert(data && name);
  for(i = 0; i < size; i++){
    if(data[i].name && !strcmp(data[i].name, name)){
      buf = ezmalloc(data[i].size + 1);
      memcpy(buf, data[i].data, data[i].size);
      buf[data[i].size] = '\0';
      return buf;
    }
  }
  return NULL;
}


char *ezhtmlescape(const char *str){
  const char *pairs[] = {
    "&", "&amp;", "<", "&lt;", ">", "&gt;", "\"", "&quot;", NULL
  };
  assert(str);
  return ezescape(str, pairs);
}


char *ezhtmlunescape(const char *str){
  const char *pairs[] = {
    "&amp;", "&", "&lt;", "<", "&gt;", ">", "&quot;", "\"", "&nbsp;", " ", NULL
  };
  assert(str);
  return ezescape(str, pairs);
}


void ezhprintf(const char *format, ...){
  va_list ap;
  char *s, *esc;
  int d;
  va_start(ap, format);
  assert(format);
  for(; *format != '\0'; format++){
    if(*format == '%'){
      format++;
      switch(*format){
      case 's':
        s = va_arg(ap, char *);
        esc = ezhtmlescape(s);
        printf("%s", esc);
        free(esc);
        break;
      case 'q':
        s = va_arg(ap, char *);
        esc = ezurlencode(s);
        printf("%s", esc);
        free(esc);
        break;
      case 'd':
        d = va_arg(ap, int);
        printf("%d", d);
        break;
      case '%':
        putchar('%');
        break;
      default:
        break;
      }
    } else {
      putchar(*format);
    }
  }
  va_end(ap);
}


EZMAP *ezbrkhttpreq(const char *str){
  EZMAP *map;
  const char *prefix = "http://";
  const char *elem = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890"
    ";/?:@&=+$,-_.!~*'()|%#";
  const char *sp;
  char *buf, *index, *rp, *server, *query, *user, *passwd, port[EZ_NUMBUFSIZ], *file;
  assert(str);
  if(!ezfwimatch(str, prefix)) return NULL;
  if(!ezstrpure(str, elem)) return NULL;
  if(ezcntchr(str, "@") > 1 || ezcntchr(str, "?") > 1) return NULL;
  sp = str + strlen(prefix);
  if(sp[0] == '\0' || strchr("#?;/:@&=", sp[0])) return NULL;
  map = eznewmap();
  buf = ezstrdup(sp);
  ezmapset(map, "url", str, TRUE);
  index = buf;
  rp = ezstrdiv(&index, "#");
  if(index) ezmapset(map, "frag", index, TRUE);
  index = rp;
  server = ezstrdiv(&index, "/");
  query = ezsprintf(NULL, "/%s", index ? index : "");
  user = passwd = NULL;
  if(strchr(server, '@')){
    index = server;
    user = ezstrdiv(&index, "@");
    server = index;
    index = user;
    user = ezstrdiv(&index, ":");
    passwd = index;
  }
  sprintf(port, "%d", 80);
  if(strchr(server, ':')){
    index = server;
    server = ezstrdiv(&index, ":");
    sprintf(port, "%d", atoi(index));
  }
  ezmapset(map, "server", server, TRUE);
  if(user) ezmapset(map, "user", user, TRUE);
  if(passwd) ezmapset(map, "passwd", passwd, TRUE);
  ezmapset(map, "port", port, TRUE);
  ezmapset(map, "query", query, TRUE);
  if(query[strlen(query)-1] != '/'){
    file = strrchr(query, '/') + 1;
    index = file;
    file = ezstrdiv(&index, "?");
    if(file[0] != '\0') ezmapset(map, "file", file, TRUE);
  }
  free(query);
  free(buf);
  return map;
}


char *ezbaseencode(const char *obj, int size){
  char *tbl = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
  char *buf, *wp;
  const unsigned char *lobj;
  int i;
  assert(obj && size >= 0);
  buf = ezmalloc(4*(size+2)/3+1);
  lobj = (const unsigned char *)obj;
  wp = buf;
  for(i = 0; i < size; i += 3){
    *wp++ = tbl[lobj[0] >> 2];
    *wp++ = tbl[((lobj[0] & 3) << 4) + (lobj[1] >> 4)];
    *wp++ = tbl[((lobj[1] & 0xf) << 2) + (lobj[2] >> 6)];
    *wp++ = tbl[lobj[2] & 0x3f];
    lobj += 3;
  }
  if(i == size + 1){
    *(wp - 1) = '=';
  } else if(i == size + 2){
    *(wp - 1) = *(wp - 2) = '=';
  }
  *wp = '\0';
  return buf;
}


char *ezbasedecode(const char *str, int *sp){
  unsigned char *obj, *wp;
  int len, cnt, bpos, i, bits, eqcnt;
  assert(str && sp);
  cnt = bpos = eqcnt = 0;
  len = strlen(str);
  wp = obj = ezmalloc(len);
  while(bpos < len && eqcnt == 0){
    bits = 0;
    for(i = 0; bpos < len && i < 4; bpos++){
      switch (str[bpos]) {
      case 'A': case 'B': case 'C': case 'D': case 'E': case 'F': case 'G': case 'H':
      case 'I': case 'J': case 'K': case 'L': case 'M': case 'N': case 'O': case 'P':
      case 'Q': case 'R': case 'S': case 'T': case 'U': case 'V': case 'W': case 'X':
      case 'Y': case 'Z':
        bits = (bits << 6) | (str[bpos] - 'A');
        i++;
        break;
      case 'a': case 'b': case 'c': case 'd': case 'e': case 'f': case 'g': case 'h':
      case 'i': case 'j': case 'k': case 'l': case 'm': case 'n': case 'o': case 'p':
      case 'q': case 'r': case 's': case 't': case 'u': case 'v': case 'w': case 'x':
      case 'y': case 'z':
        bits = (bits << 6) | (str[bpos] - 'a' + 26);
        i++;
        break;
      case '0': case '1': case '2': case '3': case '4': case '5': case '6': case '7':
      case '8': case '9':
        bits = (bits << 6) | (str[bpos] - '0' + 52);
        i++;
        break;
      case '+':
        bits = (bits << 6) | 62;
        i++;
        break;
      case '/':
        bits = (bits << 6) | 63;
        i++;
        break;
      case '=':
        bits <<= 6;
        i++;
        eqcnt++;
        break;
      default: break;
      }
    }
    if(i == 0 && bpos >= len) continue;
    switch(eqcnt){
    case 0:
      *wp++ = (bits >> 16) & 0xff;
      *wp++ = (bits >> 8) & 0xff;
      *wp++ = bits & 0xff;
      cnt += 3;
      break;
    case 1:
      *wp++ = (bits >> 16) & 0xff;
      *wp++ = (bits >> 8) & 0xff;
      cnt += 2;
      break;
    case 2:
      *wp++ = (bits >> 16) & 0xff;
      cnt += 1;
      break;
    }
  }
  *sp = cnt;
  return (char *)obj;
}


char *ezquoteencode(const char *str){
  const unsigned char *rp;
  char *buf, *wp;
  int cols;
  assert(str);
  rp = (const unsigned char *)str;
  buf = wp = ezmalloc(strlen(str) * 3 + 1);
  cols = 0;
  for(; *rp != '\0'; rp++){
    if(*rp == '=' || *rp < 0x20 || *rp > 0x7e){
      if(cols >= 72){
        wp += sprintf(wp, "=\n");
        cols = 0;
      }
      wp += sprintf(wp, "=%02X", *rp);
      cols += 3;
    } else {
      if(cols >= 74){
        wp += sprintf(wp, "=\n");
        cols = 0;
      }
      *(wp++) = *rp;
      cols++;
    }
  }
  *wp = '\0';
  return buf;
}


char *ezquotedecode(const char *str){
  char *buf, *wp;
  assert(str);
  buf = wp = ezmalloc(strlen(str) + 1);
  for(; *str != '\0'; str++){
    if(*str == '='){
      if(*(++str) == '\0') break;
      if(isxdigit((int)*str)){
        *wp = ezxdigit(*str) * 16;
        if(*(++str) == '\0') break;
        *wp += ezxdigit(*str);
        wp++;
      }
    } else {
      *wp = *str;
      wp++;
    }
  }
  *wp = '\0';
  return buf;
}


char *ezstriphtml(const char *str){
  int intag, incom;
  char *buf, *esc, *wp, *rp;
  buf = ezstrdup(str);
  intag = incom = FALSE;
  wp = buf;
  for(rp = buf; *rp != '\0'; rp++){
    if(incom){
      if(ezfwmatch(rp, "-->")){
        incom = FALSE;
        rp += 3;
      }
    } else if(intag){
      if(*rp == '>'){
        intag = FALSE;
      }
    } else {
      if(*rp == '<'){
        intag = TRUE;
      } else if(ezfwmatch(rp, "<!--")){
        incom = TRUE;
        rp += 4;
      } else {
        *(wp++) = *rp;
      }
    }
  }
  *wp = '\0';
  esc = ezhtmlunescape(buf);
  free(buf);
  return esc;
}
