/*************************************************************************************************
 * htmlstrip
 * Preprocessor for Snatcher: strips HTML data of tags.
 *                                                                              Mikio Hirabayashi
 * 
 *    ɸϤäHTMLǡΥȥȤ򳰤ơɸϤ롣
 *    ꤵ줿硢ɸϤˤ̾Υե뤫ǡɤ߹ࡣ
 *    ʸɤEUC-JPShift_JISISO-2022-JP졢EUC-JPѴƤϤ롣
 *      ˤʸɤϸᤵ롣-n åĤƼ¹ԤϡϤʸ
 *      ɤEUC-JPԤLF롣
 *    -a åˤäƼưȽͭˤȡHTMLͭΥѥʸƬ˸ʤ
 *      ˤϴ˥ץ졼ƥȤǤȤߤʤơѴԤʤ
 *    -c åĤƵưȡϤˡưȽη̤HTMLʤ 0 Ǥʤ
 *      1 ֤ƽλ롣
 *    -t åĤƵưȡHTMLtitleǤͤΤߤϤ롣-a åˤ뼫ư
 *      Ƚǥץ졼ƥȤǤȤߤʤ줿䡢titleǤʤϲϤʤ
 *************************************************************************************************/


/* ⥸塼μ */
#include "easy.h"                           /* ꥸʥɸ饤֥ */


#define CHECKLIMIT    128                   /* HTMLȽĹ */


/* Хѿ */
int normalmode = FALSE;                     /* å */
int automode = FALSE;                       /* HTMLưȽꥹå */
int checkmode = FALSE;                      /* åå */
int titlemode = FALSE;                      /* ȥ륹å */
int rv = 0;                                 /* λ */


/* ؿץȥ */
int main(int artc, char **argv);
void usage(void);
void outputtext(FILE *IN);
char *gettitle(const char* str);
int ishtml(const char *str);
void runcheck(FILE *IN);


/* ᥤ롼 */
int main(int argc, char **argv){
  int i, hit;
  FILE *IN;
  ezprogname = argv[0];
  hit = FALSE;
  for(i = 1; i < argc; i++){
    if(argv[i][0] == '-'){
      if(!strcmp(argv[i], "-n")){
        normalmode = TRUE;
      } else if(!strcmp(argv[i], "-a")){
        automode = TRUE;
      } else if(!strcmp(argv[i], "-c")){
        checkmode = TRUE;
      } else if(!strcmp(argv[i], "-t")){
        titlemode = TRUE;
      } else {
        usage();
      }
    } else {
      hit = TRUE;
      if((IN = fopen(argv[i], "r")) != NULL){
        if(checkmode){
          runcheck(IN);
        } else {
          outputtext(IN);
        }
        fclose(IN);
      } else {
        fprintf(stderr, "%s: %s: cannot open\n", ezprogname, argv[i]);
        rv = 1;
      }
    }
  }
  if(!hit){
    if(checkmode){
      runcheck(stdin);
    } else {
      outputtext(stdin);
    }
  }
  return rv;
}


/* ˡåϤƽλ */
void usage(void){
  fprintf(stderr, "%s: usage: %s [-n] [-a] [-c] [-t] [file] ...\n", ezprogname, ezprogname);
  exit(1);
}


/* ȥ꡼फǡɤ߹ǥƥȤϤ */
void outputtext(FILE *IN){
  char *str, *esc;
  EZCODETYPE ctype;
  int i;
  str = ezreadstream(IN, NULL);
  ctype = ezcharcode(str);
  switch(ctype){
  case EZ_CASCII:
    break;
  case EZ_CEUC:
    break;
  case EZ_CJUNET:
    esc = ezjunettoeuc(str);
    free(str);
    str = esc;
    break;
  case EZ_CSJIS:
    esc = ezsjistoeuc(str);
    free(str);
    str = esc;
    break;
  case EZ_CERR:
    free(str);
    rv = 1;
    return;
  }
  if(titlemode){
    if(automode && !ishtml(str)){
      rv = 1;
      free(str);
      return;
    }
    esc = gettitle(str);
    free(str);
    if(!esc){
      rv = 1;
      return;
    }
    str = esc;
    esc = ezstriphtml(str);
  } else if(!automode || ishtml(str)){
    esc = ezstriphtml(str);
    free(str);
  } else {
    esc = str;
  }
  if(normalmode){
    str = ezlineescape(esc, EZ_LUNIX);
    free(esc);
  } else {
    switch(ctype){
    case EZ_CJUNET:
      str = ezeuctojunet(esc);
      free(esc);
      break;
    case EZ_CSJIS:
      str = ezeuctosjis(esc);
      free(esc);
      break;
    default:
      str = esc;
      break;
    }
  }
  for(i = 0; str[i] != '\0'; i++){
    putchar(str[i]);
  }
  free(str);
  if(titlemode) putchar('\n');
}


/* ȥǤͤ򥢥Ȥ֤ʤ NULL ֤ */
char *gettitle(const char* str){
  const char *sp, *ep;
  char *np;
  for(sp = str; *sp != '\0'; sp++){
    if(ezfwimatch(sp, "<title")){
      for(ep = sp; *ep != '>'; ep++){
        if(*ep == '\0') return NULL;
      }
      while(!ezfwimatch(ep, "</title")){
        if(*ep == '\0') return NULL;
        ep++;
      }
      while(*ep != '>'){
        if(*ep == '\0') return NULL;
        ep++;
      }
      ep++;
      np = ezmalloc(ep - sp + 1);
      memcpy(np, sp, ep - sp);
      np[ep-sp] = '\0';
      ezascpurify(np, ' ', "");
      return np;
    }
  }
  return NULL;
}


/* HTMLݤåƽλ */
void runcheck(FILE *IN){
  char buf[CHECKLIMIT+1];
  int c, i;
  for(i = 0; i < CHECKLIMIT+1; i++){
    if((c = fgetc(IN)) == EOF) break;
    buf[i] = c;
  }
  buf[i] = '\0';
  exit(ishtml(buf) ? 0 : 1);
}


/* HTMLݤȽꤷ֤ */
int ishtml(const char *str){
  char *htmlpatterns[] = { "<!doctype", "<!--", "<html", "<head", "<title", "<body", "<p", NULL };
  int i;
  while(*str != '\0'){
    for(i = 0; htmlpatterns[i] && i < CHECKLIMIT; i++){
      if(ezfwimatch(str, htmlpatterns[i])) return TRUE;
    }
    str++;
  }
  return FALSE;
}
